import 'dart:io';

import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_upload/ocg_upload.dart';
import 'package:path/path.dart';
import 'package:uuid/uuid.dart';

class UploadHelper {
  static Future<AppResponse<VideoData>> uploadVideo(
      UploadRepository uploadRepository, VideoData fileData) async {
    String uploadedVideo, uploadedThumbnail;
    try {
      await Future.wait([
        Future(() async {
          AppResponse<String> response = await uploadRepository
              .storeFile(File(fileData.url), path: "videos/" + Uuid().v4());
          if (response.isSuccess) {
            uploadedVideo = response.data;
          }
        }),
        Future(() async {
          AppResponse<String> response = await uploadRepository.storeFile(
              File(fileData.thumbnail),
              path: "thumnbnail/" + Uuid().v4());
          if (response.isSuccess) {
            uploadedThumbnail = response.data;
          }
        }),
      ]);
      if (uploadedVideo != null) {
        return AppResponse.named(
            data: VideoData.named(
                url: uploadedVideo, thumbnail: uploadedThumbnail));
      }
      return AppResponse.named(error: "Video not uploaded");
    } catch (e) {
      return AppResponse.named(error: "${e.toString()}");
    }
  }

  static String getFileName(File file) {
    return basename(file.path);
  }
}
