import 'package:flutter/material.dart';
import 'package:ocg_upload/src/blocs/upload.dart';
import 'package:ocg_upload/src/helpers/upload.dart';

class UploadStatusWidget extends StatelessWidget {
  final UploadBloc uploadBloc;
  const UploadStatusWidget({Key key, @required this.uploadBloc})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return uploadBloc == null
        ? Container()
        : StreamBuilder<UploadState>(
            builder: (context, snapshot) {
              if (snapshot.hasData) {
                UploadState uploadState = snapshot.data;
                if ((uploadState.totalInProgress ?? []).length ==
                    (uploadState.totalDone ?? []).length) {
                  return Container();
                }
                return Material(
                  child: InkWell(
                    onTap: () {
                      showModalBottomSheet(
                          context: context,
                          builder: (context) => UploadStatusSheet(
                                uploadBloc: uploadBloc,
                              ));
                    },
                    child: Padding(
                      padding: const EdgeInsets.all(12.0),
                      child: Row(
                        children: [
                          Expanded(
                            child: Text(
                                "Uploading files ${(uploadState.totalDone ?? []).length}/${uploadState.totalInProgress.length}"),
                          ),
                          Center(
                            child: CircularProgressIndicator(),
                          )
                        ],
                      ),
                    ),
                  ),
                );
              }
              return Container();
            },
            initialData: UploadState.named(totalDone: [], totalInProgress: []),
            stream: uploadBloc.state,
          );
  }
}

class UploadStatusSheet extends StatelessWidget {
  final UploadBloc uploadBloc;
  const UploadStatusSheet({Key key, @required this.uploadBloc})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("Uploading"),
        centerTitle: true,
        elevation: 1,
        leading: IconButton(
          icon: Icon(Icons.keyboard_arrow_down),
          onPressed: () => Navigator.of(context).pop(),
        ),
      ),
      body: StreamBuilder(
        builder: (context, snapshot) {
          if (snapshot.hasData) {
            UploadState uploadState = snapshot.data;
            if ((uploadState.totalInProgress ?? []).length ==
                (uploadState.totalDone ?? []).length) {
              return Container();
            }
            List<UploadFile> inProgress = uploadState.totalInProgress;
            List<UploadResult> result = uploadState.totalDone;
            List<Widget> resultData = result.map((e) => _buildDone(e)).toList();
            List<Widget> inProgressData =
                inProgress.map((e) => _buildProgress(e)).toList();
            List<Widget> children = [...resultData, ...inProgressData];
            return ListView.builder(
              itemBuilder: (context, index) => children[index],
              shrinkWrap: true,
              itemCount: children.length,
            );
          }
          return Container();
        },
        initialData: UploadState.named(totalDone: [], totalInProgress: []),
        stream: uploadBloc.state,
      ),
    );
  }

  Widget _buildDone(UploadResult result) {
    String name = UploadHelper.getFileName(result.file);
    return Padding(
      padding: const EdgeInsets.symmetric(vertical: 8, horizontal: 16),
      child: Row(
        children: [
          Icon(
            Icons.check_circle,
            color: Colors.green,
          ),
          SizedBox(
            width: 12,
          ),
          Expanded(child: Text("$name"))
        ],
      ),
    );
  }

  Widget _buildProgress(UploadFile upload) {
    String name = UploadHelper.getFileName(upload.file);
    return Padding(
      padding: const EdgeInsets.symmetric(vertical: 8, horizontal: 16),
      child: Row(
        children: [
          Center(
            child: SizedBox(
                height: 32, width: 32, child: CircularProgressIndicator()),
          ),
          SizedBox(
            width: 12,
          ),
          Expanded(child: Text("$name"))
        ],
      ),
    );
  }
}
